@@ -19,6 +19,7 @@
 //!                "right.".roff().bold(),
 //!            ],
 //!            Some(4),
+//!            Some("optional-title")
 //!        ),
 //!        RoffNode::synopsis(
 //!                 "roffman-command",
@@ -52,10 +53,10 @@
 //! .SH "BASIC USAGE"
 //! .P
 //! This is how you create a basic paragraph using roffman\.
-//! .IP "" 4
+//! .IP optional\-title 4
 //! This line should be slightly indented to the \fBright\.\fR
 //! .SY roffman\-command
-//! This is the description of this command\. It will be displayed right next to \fIit\fR
+//! This is the description of this command\. It will be displayed right next to\fI it\fR
 //!
 //! .OP \-\-opt
 //! some simple opt
@@ -148,6 +149,7 @@ const LEFT_QUOTE: &[u8] = b"\\*(lq";
 const RIGHT_QUOTE: &[u8] = b"\\*(rq";
 const REGISTERED_SIGN: &[u8] = b"\\*R";
 const TRADEMARK_SIGN: &[u8] = b"\\*(Tm";
+const BREAK: &[u8] = b".br";
 
 #[derive(Debug)]
 /// An error type returned by the functions used in this crate.
@@ -290,8 +292,9 @@ impl Roff {
     pub fn render<W: Write>(&self, writer: &mut W) -> Result<(), RoffError> {
         self.write_title_header(writer)?;
 
+        let mut was_text = false;
         for section in &self.sections {
-            section.render(writer)?;
+            was_text = section.render(writer, was_text)?;
         }
 
         Ok(())
@@ -376,7 +379,10 @@ impl Section {
         self
     }
 
-    fn render<W: Write>(&self, writer: &mut W) -> Result<(), RoffError> {
+    fn render<W: Write>(&self, writer: &mut W, was_text: bool) -> Result<bool, RoffError> {
+        if was_text {
+            writer.write_all(ENDL)?;
+        }
         writer.write_all(SECTION_HEADER)?;
         writer.write_all(SPACE)?;
         write_quoted_if_whitespace(&self.title, writer)?;
@@ -390,11 +396,10 @@ impl Section {
 
         let mut was_text = false;
         for node in &self.nodes {
-            node.render(writer, false, was_text)?;
-            was_text = node.is_text();
+            was_text = node.render(writer, was_text)?;
         }
 
-        Ok(())
+        Ok(was_text)
     }
 }
 
@@ -471,6 +476,11 @@ impl RoffNode {
         self.0
     }
 
+    #[inline]
+    fn inner_ref(&self) -> &RoffNodeInner {
+        &self.0
+    }
+
     /// Creates a simple text node.
     pub fn text(content: impl Roffable) -> Self {
         Self(RoffNodeInner::Text(content.roff()))
@@ -491,7 +501,11 @@ impl RoffNode {
     }
 
     /// Creates a new indented paragraph node.
-    pub fn indented_paragraph<I, R>(content: I, indentation: Option<u8>) -> Self
+    pub fn indented_paragraph<I, R>(
+        content: I,
+        indentation: Option<u8>,
+        title: Option<impl Roffable>,
+    ) -> Self
     where
         I: IntoIterator<Item = R>,
         R: IntoRoffNode,
@@ -502,15 +516,15 @@ impl RoffNode {
                 .map(|item| item.into_roff().into_inner())
                 .collect(),
             indentation,
+            title: title.map(|t| t.roff()),
         })
     }
 
     /// Creates a new paragraph node with a title.
-    pub fn tagged_paragraph<I, R, T>(content: I, title: T) -> Self
+    pub fn tagged_paragraph<I, R>(content: I, title: impl Roffable) -> Self
     where
         I: IntoIterator<Item = R>,
         R: IntoRoffNode,
-        T: Roffable,
     {
         Self(RoffNodeInner::TaggedParagraph {
             content: content
@@ -583,6 +597,23 @@ impl RoffNode {
     pub fn trademark_sign() -> Self {
         Self(RoffNodeInner::TrademarkSign)
     }
+
+    /// Nest nodes by indenting all of the nodes inside.
+    pub fn nested<I, R>(nodes: I) -> Self
+    where
+        I: IntoIterator<Item = R>,
+        R: IntoRoffNode,
+    {
+        Self(RoffNodeInner::Nested(
+            nodes.into_iter().map(R::into_roff).collect(),
+        ))
+    }
+
+    /// Breaks the line in text. Use this instead of adding raw `\n` characters to actually render
+    /// linebreaks.
+    pub fn linebreak() -> Self {
+        Self(RoffNodeInner::Break)
+    }
 }
 
 #[derive(Clone, Debug)]
@@ -632,6 +663,7 @@ enum RoffNodeInner {
     IndentedParagraph {
         content: Vec<RoffNodeInner>,
         indentation: Option<u8>,
+        title: Option<RoffText>,
     },
     /// Paragraph with a title.
     TaggedParagraph {
@@ -657,28 +689,12 @@ enum RoffNodeInner {
     LeftQuote,
     RightQuote,
     TrademarkSign,
+    Nested(Vec<RoffNode>),
+    Break,
 }
 
 impl RoffNodeInner {
-    /// Returns `true` if the node is the [`RoffNode::Text`](RoffNode::Text) variant.
-    fn is_text(&self) -> bool {
-        matches!(self, &RoffNodeInner::Text(_))
-    }
-
-    fn is_nestable(&self) -> bool {
-        !self.is_text()
-    }
-
-    fn render<W: Write>(
-        &self,
-        writer: &mut W,
-        nested: bool,
-        was_text: bool,
-    ) -> Result<(), RoffError> {
-        if nested {
-            writer.write_all(ENDL)?;
-            writer.write_all(NESTED_START)?;
-        }
+    fn render<W: Write>(&self, writer: &mut W, mut was_text: bool) -> Result<bool, RoffError> {
         match self {
             RoffNodeInner::Text(text) => {
                 let styled = match text.style {
@@ -697,6 +713,7 @@ impl RoffNodeInner {
                 if styled {
                     writer.write_all(FONT_END)?;
                 }
+                was_text = true;
             }
             RoffNodeInner::Paragraph(content) => {
                 if was_text {
@@ -704,31 +721,36 @@ impl RoffNodeInner {
                 }
                 writer.write_all(PARAGRAPH)?;
                 writer.write_all(ENDL)?;
-                let mut was_text_node = false;
                 for node in content {
-                    node.render(writer, node.is_nestable(), was_text_node)?;
-                    was_text_node = node.is_text();
+                    was_text = node.render(writer, was_text)?;
                 }
-                writer.write_all(ENDL)?;
             }
             RoffNodeInner::IndentedParagraph {
                 content,
                 indentation,
+                title,
             } => {
                 if was_text {
                     writer.write_all(ENDL)?;
                 }
                 writer.write_all(INDENTED_PARAGRAPH)?;
                 if let Some(indentation) = indentation {
-                    writer.write_all(format!(" \"\" {}", indentation).as_bytes())?;
+                    writer.write_all(SPACE)?;
+                    if let Some(title) = title {
+                        write_quoted_if_whitespace(title, writer)?;
+                    } else {
+                        writer.write_all(QUOTE)?;
+                        writer.write_all(QUOTE)?;
+                    }
+                    writer.write_all(SPACE)?;
+                    indentation.roff().render(writer)?;
                 }
                 writer.write_all(ENDL)?;
-                let mut was_text_node = false;
                 for node in content {
-                    node.render(writer, node.is_nestable(), was_text_node)?;
-                    was_text_node = node.is_text();
+                    was_text = node.render(writer, was_text)?;
                 }
                 writer.write_all(ENDL)?;
+                was_text = false;
             }
             RoffNodeInner::TaggedParagraph {
                 content,
@@ -742,12 +764,11 @@ impl RoffNodeInner {
                 tag.render(writer)?;
                 writer.write_all(ENDL)?;
 
-                let mut was_text_node = false;
                 for node in content {
-                    node.render(writer, node.is_nestable(), was_text_node)?;
-                    was_text_node = node.is_text();
+                    was_text = node.render(writer, was_text)?;
                 }
                 writer.write_all(ENDL)?;
+                was_text = false;
             }
             RoffNodeInner::Example(content) => {
                 if was_text {
@@ -761,6 +782,7 @@ impl RoffNodeInner {
                 writer.write_all(ENDL)?;
                 writer.write_all(EXAMPLE_END)?;
                 writer.write_all(ENDL)?;
+                was_text = false;
             }
             RoffNodeInner::Synopsis {
                 command,
@@ -799,6 +821,7 @@ impl RoffNodeInner {
                 }
                 writer.write_all(SYNOPSIS_END)?;
                 writer.write_all(ENDL)?;
+                was_text = false;
             }
             RoffNodeInner::Url { address, name } => {
                 if was_text {
@@ -809,9 +832,12 @@ impl RoffNodeInner {
                 address.render(writer)?;
                 writer.write_all(ENDL)?;
                 name.render(writer)?;
-                writer.write_all(ENDL)?;
+                if !name.content.is_empty() {
+                    writer.write_all(ENDL)?;
+                }
                 writer.write_all(URL_END)?;
                 writer.write_all(ENDL)?;
+                was_text = false;
             }
             RoffNodeInner::Email { address, name } => {
                 if was_text {
@@ -822,22 +848,43 @@ impl RoffNodeInner {
                 address.render(writer)?;
                 writer.write_all(ENDL)?;
                 name.render(writer)?;
-                writer.write_all(ENDL)?;
+                if !name.content.is_empty() {
+                    writer.write_all(ENDL)?;
+                }
                 writer.write_all(MAIL_END)?;
                 writer.write_all(ENDL)?;
+                was_text = false;
             }
             RoffNodeInner::RegisteredSign => writer.write_all(REGISTERED_SIGN)?,
             RoffNodeInner::LeftQuote => writer.write_all(LEFT_QUOTE)?,
             RoffNodeInner::RightQuote => writer.write_all(RIGHT_QUOTE)?,
             RoffNodeInner::TrademarkSign => writer.write_all(TRADEMARK_SIGN)?,
+            RoffNodeInner::Nested(nodes) => {
+                if was_text {
+                    writer.write_all(ENDL)?;
+                }
+                writer.write_all(NESTED_START)?;
+                writer.write_all(ENDL)?;
+                was_text = false;
+                for node in nodes {
+                    was_text = node.inner_ref().render(writer, was_text)?;
+                }
+
+                if was_text {
+                    writer.write_all(ENDL)?;
+                }
+                writer.write_all(NESTED_END)?;
+                writer.write_all(ENDL)?;
+                was_text = false;
+            }
+            RoffNodeInner::Break => {
+                writer.write_all(ENDL)?;
+                writer.write_all(BREAK)?;
+                writer.write_all(ENDL)?;
+            }
         }
 
-        if nested {
-            writer.write_all(ENDL)?;
-            writer.write_all(NESTED_END)?;
-        }
-
-        Ok(())
+        Ok(was_text)
     }
 }
 
@@ -895,6 +942,12 @@ impl Roffable for &&str {
     }
 }
 
+impl Roffable for std::borrow::Cow<'_, str> {
+    fn roff(&self) -> RoffText {
+        self.as_ref().roff()
+    }
+}
+
 impl Roffable for RoffText {
     fn roff(&self) -> RoffText {
         self.clone()
@@ -916,7 +969,7 @@ mod tests {
         let roff = Roff::new("test", SectionNumber::UserCommands)
             .section(
                 "test section 1",
-                vec![RoffNode::paragraph(vec![
+                [RoffNode::paragraph([
                     "this is some very ".roff(),
                     "special".roff().bold(),
                     " text".roff(),
@@ -924,22 +977,43 @@ mod tests {
             )
             .section(
                 "test section 2",
-                vec![RoffNode::indented_paragraph(
-                    vec![
+                [RoffNode::indented_paragraph(
+                    [
                         "Lorem ipsum".roff().italic(),
                         " dolor sit amet, consectetur adipiscing elit. Vivamus quis malesuada eros.".roff()
                             .roff(),
                     ],
                     Some(4),
+                    None::<&str>
                 )],
             )
             .section(
                 "test section 3",
-                vec![RoffNode::tagged_paragraph(
-                    vec!["tagged paragraph with some content".roff()],
+                [RoffNode::tagged_paragraph(
+                    ["tagged paragraph with some content".roff()],
                     "paragraph title".roff().bold(),
                 )],
-            );
+            )
+            .section(
+                "test section 4",
+                [
+                RoffNode::indented_paragraph(
+                    [
+                        "Indented paragraph with a title",
+                    ],
+                    Some(4),
+                    Some("Paragraph title with spaces")
+                ),
+                RoffNode::indented_paragraph(
+                    [
+                        "Another indented paragraph",
+                    ],
+                    Some(2),
+                    Some("title-no-spaces")
+                )
+                ],
+            )
+            ;
 
         let rendered = roff.to_string().unwrap();
         assert_eq!(
@@ -954,6 +1028,11 @@ this is some very \fBspecial\fR text
 .TP
 \fBparagraph title\fR
 tagged paragraph with some content
+.SH "test section 4"
+.IP "Paragraph title with spaces" 4
+Indented paragraph with a title
+.IP title\-no\-spaces 2
+Another indented paragraph
 "#,
             rendered
         )
@@ -964,17 +1043,18 @@ tagged paragraph with some content
         let roff = Roff::new("test", SectionNumber::UserCommands).add_section(
             Section::new(
                 "BASE SECTION",
-                vec![
-                    RoffNode::paragraph(vec![
+                [
+                    RoffNode::paragraph([
                         RoffNode::text("some text in first paragraph."),
-                        RoffNode::paragraph(vec![
+                        RoffNode::nested([RoffNode::paragraph([
                             RoffNode::text("some nested paragraph"),
-                            RoffNode::paragraph(vec![RoffNode::text(
+                            RoffNode::nested([RoffNode::paragraph([RoffNode::text(
                                 "some doubly nested paragraph",
-                            )]),
-                        ]),
+                            )])]),
+                            RoffNode::text("some text after nested para"),
+                        ])]),
                     ]),
-                    RoffNode::paragraph(vec!["back two levels left", " without roffs"]),
+                    RoffNode::paragraph(["back two levels left", " without roffs"]),
                 ],
             )
             .subtitle("with some subtitle..."),
@@ -982,6 +1062,7 @@ tagged paragraph with some content
 
         let rendered = roff.to_string().unwrap();
         assert_eq!(
+            rendered,
             r#".TH test 1
 .SH "BASE SECTION"
 .SS "with some subtitle\.\.\."
@@ -993,14 +1074,11 @@ some nested paragraph
 .RS
 .P
 some doubly nested paragraph
-
 .RE
-
+some text after nested para
 .RE
 .P
-back two levels left without roffs
-"#,
-            rendered
+back two levels left without roffs"#,
         )
     }
 
@@ -1094,7 +1172,11 @@ with \-l, scale sizes by SIZE when printing them
                 RoffNode::url("GitHub", "https://github.com/vv9k/roffman"),
                 RoffNode::url("crates.io", "https://crates.io/crates/roffman"),
                 RoffNode::url("docs.rs", "https://docs.rs/roffman"),
+                RoffNode::url("", "https://docs.rs/roffman"),
+                RoffNode::url("", ""),
                 RoffNode::email("John Test", "test@invalid.domain"),
+                RoffNode::email("", "test@invalid.domain"),
+                RoffNode::email("", ""),
             ],
         );
 
@@ -1111,9 +1193,17 @@ crates\.io
 .UR https://docs\.rs/roffman
 docs\.rs
 .UE
+.UR https://docs\.rs/roffman
+.UE
+.UR 
+.UE
 .MT test@invalid\.domain
 John Test
 .ME
+.MT test@invalid\.domain
+.ME
+.MT 
+.ME
 "#,
             rendered
         )
@@ -1142,4 +1232,59 @@ John Test
             rendered
         )
     }
+
+    #[test]
+    fn section_after_text_renders() {
+        let roff = Roff::new("test-sections", SectionNumber::Miscellaneous)
+            .section("TEXTS", vec![RoffNode::text("this is some example text.")])
+            .section(
+                "NEXT",
+                vec![
+                    RoffNode::text("this is some example text on second section.\n"),
+                    RoffNode::text("this is some example.\n"),
+                    RoffNode::text("this is some example text."),
+                ],
+            )
+            .section("THIRD", vec![RoffNode::text("this is some example text.")]);
+
+        let rendered = roff.to_string().unwrap();
+        assert_eq!(
+            r#".TH test\-sections 7
+.SH TEXTS
+this is some example text\.
+.SH NEXT
+this is some example text on second section\.
+this is some example\.
+this is some example text\.
+.SH THIRD
+this is some example text\."#,
+            rendered
+        )
+    }
+
+    #[test]
+    fn breaks_line() {
+        let roff = Roff::new("test-breaks", SectionNumber::Miscellaneous).section(
+            "BREAKS",
+            vec![
+                RoffNode::text("this is some example text."),
+                RoffNode::linebreak(),
+                RoffNode::text("this is some example text on second line."),
+                RoffNode::linebreak(),
+                RoffNode::text("this is some example text on third line."),
+            ],
+        );
+
+        let rendered = roff.to_string().unwrap();
+        assert_eq!(
+            rendered,
+            r#".TH test\-breaks 7
+.SH BREAKS
+this is some example text\.
+.br
+this is some example text on second line\.
+.br
+this is some example text on third line\."#
+        )
+    }
 }
